@@ -7,7 +7,7 @@ package AnyEvent::Redis::RipeRedis;
 
 use base qw( Exporter );
 
-our $VERSION = '1.45_04';
+our $VERSION = '1.47_01';
 
 use AnyEvent;
 use AnyEvent::Handle;
@@ -16,62 +16,45 @@ use Scalar::Util qw( looks_like_number weaken );
 use Digest::SHA qw( sha1_hex );
 use Carp qw( croak );
 
-BEGIN {
-  our @EXPORT_OK = qw(
-    E_CANT_CONN
-    E_LOADING_DATASET
-    E_IO
-    E_CONN_CLOSED_BY_REMOTE_HOST
-    E_CONN_CLOSED_BY_CLIENT
-    E_NO_CONN
-    E_OPRN_ERROR
-    E_UNEXPECTED_DATA
-    E_NO_SCRIPT
-    E_READ_TIMEDOUT
-    E_BUSY
-    E_MASTER_DOWN
-    E_MISCONF
-    E_READONLY
-    E_OOM
-    E_EXEC_ABORT
-    E_NO_AUTH
-    E_WRONG_TYPE
-    E_NO_REPLICAS
-    E_BUSY_KEY
-  );
+my %ERROR_CODES;
 
-  our %EXPORT_TAGS = (
-    err_codes => \@EXPORT_OK,
+BEGIN {
+  %ERROR_CODES = (
+    E_CANT_CONN                  => 1,
+    E_LOADING_DATASET            => 2,
+    E_IO                         => 3,
+    E_CONN_CLOSED_BY_REMOTE_HOST => 4,
+    E_CONN_CLOSED_BY_CLIENT      => 5,
+    E_NO_CONN                    => 6,
+    E_OPRN_ERROR                 => 9,
+    E_UNEXPECTED_DATA            => 10,
+    E_NO_SCRIPT                  => 11,
+    E_READ_TIMEDOUT              => 12,
+    E_BUSY                       => 13,
+    E_MASTER_DOWN                => 14,
+    E_MISCONF                    => 15,
+    E_READONLY                   => 16,
+    E_OOM                        => 17,
+    E_EXEC_ABORT                 => 18,
+    E_NO_AUTH                    => 19,
+    E_WRONG_TYPE                 => 20,
+    E_NO_REPLICAS                => 21,
+    E_BUSY_KEY                   => 22,
   );
 }
 
+BEGIN {
+  our @EXPORT_OK   = keys %ERROR_CODES;
+  our %EXPORT_TAGS = ( err_codes => \@EXPORT_OK, );
+}
+
 use constant {
   # Default values
   D_HOST     => 'localhost',
   D_PORT     => 6379,
   D_DB_INDEX => 0,
 
-  # Error codes
-  E_CANT_CONN                  => 1,
-  E_LOADING_DATASET            => 2,
-  E_IO                         => 3,
-  E_CONN_CLOSED_BY_REMOTE_HOST => 4,
-  E_CONN_CLOSED_BY_CLIENT      => 5,
-  E_NO_CONN                    => 6,
-  E_OPRN_ERROR                 => 9,
-  E_UNEXPECTED_DATA            => 10,
-  E_NO_SCRIPT                  => 11,
-  E_READ_TIMEDOUT              => 12,
-  E_BUSY                       => 13,
-  E_MASTER_DOWN                => 14,
-  E_MISCONF                    => 15,
-  E_READONLY                   => 16,
-  E_OOM                        => 17,
-  E_EXEC_ABORT                 => 18,
-  E_NO_AUTH                    => 19,
-  E_WRONG_TYPE                 => 20,
-  E_NO_REPLICAS                => 21,
-  E_BUSY_KEY                   => 22,
+  %ERROR_CODES,
 
   # Operation status
   S_NEED_PERFORM => 1,
@@ -130,10 +113,11 @@ sub new {
 
   my $self = ref( $proto ) ? $proto : bless {}, $proto;
 
-  $self->{host}      = $params{host} || D_HOST;
-  $self->{port}      = $params{port} || D_PORT;
-  $self->{password}  = $params{password};
-  $self->{database}  = defined $params{database} ? $params{database} : D_DB_INDEX;
+  $self->{host} = $params{host} || D_HOST;
+  $self->{port} = $params{port} || D_PORT;
+  $self->{password} = $params{password};
+  $self->{database}
+      = defined $params{database} ? $params{database} : D_DB_INDEX;
   $self->{reconnect} = exists $params{reconnect} ? $params{reconnect} : 1;
   $self->{on_connect}       = $params{on_connect};
   $self->{on_disconnect}    = $params{on_disconnect};
@@ -162,8 +146,8 @@ sub new {
   $self->{_tmp_queue}      = [];
   $self->{_process_queue}  = [];
   $self->{_txn_lock}       = 0;
-  $self->{_subs}           = {};
-  $self->{_subs_num}       = 0;
+  $self->{_sub_map}        = {};
+  $self->{_sub_cnt}        = 0;
 
   unless ( $self->{_lazy_conn_st} ) {
     $self->_connect();
@@ -321,8 +305,10 @@ sub selected_database {
       if ( @_ ) {
         my $timeout = shift;
 
-        if ( defined $timeout && ( !looks_like_number( $timeout ) || $timeout < 0 ) ) {
-          croak ucfirst( $name_pref ) . ' timeout must be a positive number';
+        if ( defined $timeout
+          && ( !looks_like_number($timeout) || $timeout < 0 ) )
+        {
+          croak ucfirst($name_pref) . ' timeout must be a positive number';
         }
         $self->{$name} = $timeout;
       }
@@ -780,17 +766,20 @@ sub _process_reply {
         ? ( "Operation '$cmd->{kwd}' completed with errors.", $err_code, $data )
         : $data, $err_code );
   }
-  elsif ( $self->{_subs_num} > 0 && ref( $data ) && exists $MSG_TYPES{ $data->[0] } ) {
-    my $on_msg = $self->{_subs}{ $data->[1] };
+  elsif ( $self->{_sub_cnt} > 0
+    && ref( $data ) && exists $MSG_TYPES{ $data->[0] } )
+  {
+    my $cmd = $self->{_sub_map}{ $data->[1] };
 
-    unless ( defined $on_msg ) {
+    unless ( defined $cmd ) {
       $self->_disconnect( "Don't know how process published message."
           . " Unknown channel or pattern '$data->[1]'.", E_UNEXPECTED_DATA );
 
       return;
     }
 
-    $on_msg->( $data->[0] eq 'pmessage' ? @{$data}[ 2, 3, 1 ] : @{$data}[ 1, 2 ] );
+    $cmd->{on_message}->( $data->[0] eq 'pmessage'
+        ? @{$data}[ 2, 3, 1 ] : @{$data}[ 1, 2 ] );
   }
   else {
     my $cmd = $self->{_process_queue}[0];
@@ -851,13 +840,13 @@ sub _process_cmd_success {
       shift @{$data};
 
       if ( exists $SUB_CMDS{$kwd} ) {
-        $self->{_subs}{ $data->[0] } = $cmd->{on_message};
+        $self->{_sub_map}{ $data->[0] } = $cmd;
       }
       else { # unsubscribe or punsubscribe
-        delete $self->{_subs}{ $data->[0] };
+        delete $self->{_sub_map}{ $data->[0] };
       }
 
-      $self->{_subs_num} = $data->[1];
+      $self->{_sub_cnt} = $data->[1];
     }
     elsif ( $kwd eq 'info' ) {
       $data = $self->_parse_info( $data );
@@ -890,7 +879,9 @@ sub _parse_info {
 
 ####
 sub _disconnect {
-  my $self = shift;
+  my $self     = shift;
+  my $err_msg  = shift;
+  my $err_code = shift;
 
   my $was_connected = $self->{_connected};
 
@@ -904,7 +895,7 @@ sub _disconnect {
   $self->{_ready_to_write} = 0;
   $self->{_txn_lock}       = 0;
 
-  $self->_abort_all( @_ );
+  $self->_abort_all( $err_msg, $err_code );
 
   if ( $was_connected && defined $self->{on_disconnect} ) {
     $self->{on_disconnect}->();
@@ -925,24 +916,35 @@ sub _abort_all {
     @{ $self->{_input_queue} },
   );
 
+  my $sub_map = $self->{_sub_map};
+  my $sub_cnt = $self->{_sub_cnt};
+
   $self->{_input_queue}   = [];
   $self->{_tmp_queue}     = [];
   $self->{_process_queue} = [];
-  $self->{_subs}          = {};
-  $self->{_subs_num}      = 0;
+  $self->{_sub_map}       = {};
+  $self->{_sub_cnt}       = 0;
 
   if ( !defined $err_msg && @unfin_cmds ) {
     $err_msg  = 'Connection closed by client prematurely.';
     $err_code = E_CONN_CLOSED_BY_CLIENT;
   }
+
   if ( defined $err_msg ) {
-    if ( $err_code == E_CANT_CONN && defined $self->{on_connect_error} ) {
+    if ( defined $self->{on_connect_error} && $err_code == E_CANT_CONN ) {
       $self->{on_connect_error}->( $err_msg );
     }
     else {
       $self->{on_error}->( $err_msg, $err_code );
     }
 
+    if ( $sub_cnt > 0 && $err_code != E_CONN_CLOSED_BY_CLIENT ) {
+      foreach my $sub_key ( keys %{$sub_map} ) {
+        my $cmd = $sub_map->{$sub_key};
+        $self->_process_cmd_error( $cmd, "Subscription '$sub_key' lost: "
+            . $err_msg, $err_code );
+      }
+    }
     foreach my $cmd ( @unfin_cmds ) {
       $self->_process_cmd_error( $cmd, "Operation '$cmd->{kwd}' aborted: "
           . $err_msg, $err_code );
@@ -1165,12 +1167,12 @@ Server port (default: 6379)
 
 =item password => $password
 
-If the password is specified, then the C<AUTH> command is sent to the server
+If the password is specified, the C<AUTH> command is sent to the server
 after connection.
 
 =item database => $index
 
-Database index. If the index is specified, then the client is switched to
+Database index. If the index is specified, the client is switched to
 the specified database after connection. You can also switch to another database
 after connection by using C<SELECT> command. The client remembers last selected
 database after reconnection.
@@ -1187,7 +1189,7 @@ Not set by default.
 
 Timeout, within which the client will be wait the connection establishment to
 the Redis server. If the client could not connect to the server after specified
-timeout, then the C<on_error> or C<on_connect_error> callback is called. In case,
+timeout, the C<on_error> or C<on_connect_error> callback is called. In case
 when C<on_error> callback is called, C<E_CANT_CONN> error code is passed to
 callback in the second argument. The timeout specifies in seconds and can
 contain a fractional part.
@@ -1200,7 +1202,7 @@ By default the client use kernel's connection timeout.
 
 Timeout, within which the client will be wait a response on a command from the
 Redis server. If the client could not receive a response from the server after
-specified timeout, then the client close connection and call C<on_error> callback
+specified timeout, the client close connection and call C<on_error> callback
 with the C<E_READ_TIMEDOUT> error code. The timeout is specifies in seconds
 and can contain a fractional part.
 
@@ -1210,7 +1212,7 @@ Not set by default.
 
 =item lazy => $boolean
 
-If enabled, the connection establishes at time, when you will send the first
+If enabled, the connection establishes at time when you will send the first
 command to the server. By default the connection establishes after calling of
 the C<new> method.
 
@@ -1219,8 +1221,8 @@ Disabled by default.
 =item reconnect => $boolean
 
 If the connection to the Redis server was lost and the parameter C<reconnect> is
-TRUE, then the client try to restore the connection on a next command executuion.
-The client try to reconnect only once and if it fails, then is called the
+TRUE, the client try to restore the connection on a next command executuion.
+The client try to reconnect only once and if it fails, is called the
 C<on_error> callback. If you need several attempts of the reconnection, just
 retry a command from the C<on_error> callback as many times, as you need. This
 feature made the client more responsive.
@@ -1262,14 +1264,14 @@ Disabled by default.
 
 =item on_connect => $cb->()
 
-The C<on_connect> callback is called, when the connection is successfully
+The C<on_connect> callback is called when the connection is successfully
 established.
 
 Not set by default.
 
 =item on_disconnect => $cb->()
 
-The C<on_disconnect> callback is called, when the connection is closed by any
+The C<on_disconnect> callback is called when the connection is closed by any
 reason.
 
 Not set by default.
@@ -1277,8 +1279,8 @@ Not set by default.
 =item on_connect_error => $cb->( $err_msg )
 
 The C<on_connect_error> callback is called, when the connection could not be
-established. If this collback isn't specified, then the common C<on_error>
-callback is called with the C<E_CANT_CONN> error code.
+established. If this collback isn't specified, the common C<on_error> callback
+is called with the C<E_CANT_CONN> error code.
 
 Not set by default.
 
@@ -1373,18 +1375,18 @@ The full list of the Redis commands can be found here: L<http://redis.io/command
 
 =item on_done => $cb->( [ $data ] )
 
-The C<on_done> callback is called, when the current operation was completed
+The C<on_done> callback is called when the current operation was completed
 successfully.
 
 =item on_error => $cb->( $err_msg, $err_code )
 
-The C<on_error> callback is called, when some error occurred.
+The C<on_error> callback is called when some error occurred.
 
 =item on_reply => $cb->( [ $data ] [, $err_msg, $err_code ] )
 
 Since version 1.300 of the client you can specify single, C<on_reply> callback,
 instead of two, C<on_done> and C<on_error> callbacks. The C<on_reply> callback
-is called in both cases: when operation was completed successfully and when some
+is called in both cases: when operation was completed successfully or when some
 error occurred. In first case to callback is passed only reply data. In second
 case to callback is passed three arguments: The C<undef> value or reply data
 with error objects (see below), error message and error code.
@@ -1407,10 +1409,10 @@ Executes all previously queued commands in a transaction and restores the
 connection state to normal. When using C<WATCH>, C<EXEC> will execute commands
 only if the watched keys were not modified.
 
-If after execution of C<EXEC> command at least one operation fails, then
-either C<on_error> or C<on_reply> callback is called and in addition to error
-message and error code to callback is passed reply data, which contain replies
-of successful operations and error objects for each failed operation. To
+If after execution of C<EXEC> command at least one operation fails, the
+C<on_error> or C<on_reply> callback is called and in addition to error message
+and error code to callback is passed reply data, which contain replies of
+successful operations and error objects for each failed operation. To
 C<on_error> callback reply data is passed in third argument and to C<on_reply>
 callback in first argument. Error object is an instance of class
 C<AnyEvent::Redis::RipeRedis::Error> and has two methods: C<message()> to get
@@ -1506,8 +1508,11 @@ L<http://redis.io/topics/pubsub>
 Subscribes the client to the specified channels.
 
 Once the client enters the subscribed state it is not supposed to issue any
-other commands, except for additional C<SUBSCRIBE>, C<PSUBSCRIBE>, C<UNSUBSCRIBE>
-and C<PUNSUBSCRIBE> commands.
+other commands, except for additional C<SUBSCRIBE>, C<PSUBSCRIBE>, C<UNSUBSCRIBE>,
+C<PUNSUBSCRIBE> and C<QUIT> commands.
+
+If the connection was lost when the client has active subscriptions, the C<on_error>
+or C<on_reply> callback in appropriate C<(p)subscribe> method is called.
 
   $redis->subscribe( qw( ch_foo ch_bar ),
     { on_reply => sub {
@@ -1574,21 +1579,21 @@ and C<PUNSUBSCRIBE> commands.
 
 =item on_done => $cb->( $ch_name, $sub_num )
 
-The C<on_done> callback is called on every specified channel, when the
+The C<on_done> callback is called on every specified channel when the
 subscription operation was completed successfully.
 
 =item on_message => $cb->( $ch_name, $msg )
 
-The C<on_message> callback is called, when a published message was received.
+The C<on_message> callback is called when a published message was received.
 
 =item on_error => $cb->( $err_msg, $err_code )
 
-The C<on_error> callback is called, if the subscription operation fails.
+The C<on_error> callback is called if the subscription operation fails.
 
 =item on_reply => $cb->( [ $data ] [, $err_msg, $err_code ] )
 
 The C<on_reply> callback is called in both cases: when the subscription operation
-was completed successfully and when subscription operation fails. In first case
+was completed successfully or when subscription operation fails. In first case
 C<on_reply> callback is called on every specified channel. Information about
 channel name and subscription number is passed to callback in first argument as
 an array reference.
@@ -1667,21 +1672,21 @@ Subscribes the client to the given patterns.
 
 =item on_done => $cb->( $ch_pattern, $sub_num )
 
-The C<on_done> callback is called on every specified pattern, when the
+The C<on_done> callback is called on every specified pattern when the
 subscription operation was completed successfully.
 
 =item on_message => $cb->( $ch_name, $msg, $ch_pattern )
 
-The C<on_message> callback is called, when published message was received.
+The C<on_message> callback is called when published message was received.
 
 =item on_error => $cb->( $err_msg, $err_code )
 
-The C<on_error> callback is called, if the subscription operation fails.
+The C<on_error> callback is called if the subscription operation fails.
 
 =item on_reply => $cb->( [ $data ] [, $err_msg, $err_code ] )
 
 The C<on_reply> callback is called in both cases: when the subscription
-operation was completed successfully and when subscription operation fails.
+operation was completed successfully or when subscription operation fails.
 In first case C<on_reply> callback is called on every specified pattern.
 Information about channel pattern and subscription number is passed to callback
 in first argument as an array reference.
@@ -1735,17 +1740,17 @@ channel will be sent to the client.
 
 =item on_done => $cb->( $ch_name, $sub_num )
 
-The C<on_done> callback is called on every specified channel, when the
+The C<on_done> callback is called on every specified channel when the
 unsubscription operation was completed successfully.
 
 =item on_error => $cb->( $err_msg, $err_code )
 
-The C<on_error> callback is called, if the unsubscription operation fails.
+The C<on_error> callback is called if the unsubscription operation fails.
 
 =item on_reply => $cb->( [ $data ] [, $err_msg, $err_code ] )
 
 The C<on_reply> callback is called in both cases: when the unsubscription
-operation was completed successfully and when unsubscription operation fails.
+operation was completed successfully or when unsubscription operation fails.
 In first case C<on_reply> callback is called on every specified channel.
 Information about channel name and number of remaining subscriptions is passed
 to callback in first argument as an array reference.
@@ -1795,17 +1800,17 @@ pattern will be sent to the client.
 
 =item on_done => $cb->( $ch_name, $sub_num )
 
-The C<on_done> callback is called on every specified pattern, when the
+The C<on_done> callback is called on every specified pattern when the
 unsubscription operation was completed successfully.
 
 =item on_error => $cb->( $err_msg, $err_code )
 
-The C<on_error> callback is called, if the unsubscription operation fails.
+The C<on_error> callback is called if the unsubscription operation fails.
 
 =item on_reply => $cb->( [ $data ] [, $err_msg, $err_code ] )
 
 The C<on_reply> callback is called in both cases: when the unsubscription
-operation was completed successfully and when unsubscription operation fails.
+operation was completed successfully or when unsubscription operation fails.
 In first case C<on_reply> callback is called on every specified pattern.
 Information about channel pattern and number of remaining subscriptions is
 passed to callback in first argument as an array reference.
@@ -1863,9 +1868,9 @@ instead.
 Be care, passing a different Lua scripts to C<eval_cached()> method every time
 cause memory leaks.
 
-If Lua script returns multi-bulk reply with at least one error reply, then
-either C<on_error> or C<on_reply> callback is called and in addition to error
-message and error code to callback is passed reply data, which contain successful
+If Lua script returns multi-bulk reply with at least one error reply, the
+C<on_error> or C<on_reply> callback is called and in addition to error message
+and error code to callback is passed reply data, which contain successful
 replies and error objects for each error reply, as well as described for C<EXEC>
 command.
 
@@ -2144,7 +2149,7 @@ Ivan Kruglov
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (c) 2012-2014, Eugene Ponizovsky, E<lt>ponizovsky@gmail.comE<gt>.
+Copyright (c) 2012-2015, Eugene Ponizovsky, E<lt>ponizovsky@gmail.comE<gt>.
 All rights reserved.
 
 This module is free software; you can redistribute it and/or modify it under
